package cn.com.qmth.dp.examcloud.oe.modules.debug;

import cn.com.qmth.dp.examcloud.oe.entity.question.QuesOption;
import cn.com.qmth.dp.examcloud.oe.entity.question.Question;
import cn.com.qmth.dp.examcloud.oe.modules.export_course_questions_diff_count.QuestionTypeCount;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.core.questions.service.bean.dto.GenPaperDto;
import cn.com.qmth.examcloud.core.questions.service.impl.GenPaperService;
import org.apache.commons.lang3.StringUtils;
import org.bson.types.ObjectId;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.nodes.Node;
import org.jsoup.nodes.TextNode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationOperation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class DebugService {
    


    public static final String QUESOPS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

    public static final String ELEMENT_TYPE_RID = "imageId";

    public static final String ELEMENT_TYPE_TEXT = "text";

    public static final String ELEMENT_TYPE_IMG = "image";
    
    public static final String ELEMENT_TYPE_DOCTAG = "doctag";
    @Autowired
    GenPaperService genPaperService;
	@Autowired
	MongoTemplate mongoTemplate;

	public void start() {
		System.out.println("start *********************");
		GenPaperDto d=new GenPaperDto();
		d.setCourseNo("061001");
		d.setGenNumber(1);
		List<String> pids=new ArrayList<>();
		pids.add("62b965233c66227eeadbdd68");
		d.setPaperIds(pids);
		d.setPaperName("test1");
		d.setPaperStructId("656f2be8267c7c21db09b990");
		genPaperService.genPaper(d);
//		Query query = new Query();
//		query.addCriteria(Criteria.where("paper.$id").is(new ObjectId("64ddd05a391d2033f36b0a20")));
//		List<PaperDetailUnit> ps = mongoTemplate.find(query, PaperDetailUnit.class, "paperDetailUnit");
//		ExportPaperUtil.disposePaperDoc(ps);
//		for(PaperDetailUnit p:ps) {
//			disposeQuestion(p.getQuestion());
//		}
//		List<QuestionTypeCount> ret=countByType("WB7030", "17351");
//		System.out.println();
	}
	
	private List<QuestionTypeCount> countByType(String courseCode, String rootOrgId) {
        Date day=DateUtil.parse("2023-08-21 00:00:00", "yyyy-MM-dd HH:mm:ss");
		List<AggregationOperation> operations = new ArrayList<>();
		operations.add(Aggregation.match(
				Criteria.where("orgId").is(rootOrgId)
				.and("course.code").is(courseCode)
				.and("creationBy").is(646523L)
				.and("creationDate").gt(day)
					)
				);
		operations.add(Aggregation.group("questionType", "difficulty").count().as("count"));
		operations.add(Aggregation.project("count").andInclude("questionType").andInclude("difficulty"));
		Aggregation aggregation = Aggregation.newAggregation(operations);
		AggregationResults<QuestionTypeCount> outputTypeCount = mongoTemplate.aggregate(aggregation, Question.class,
				QuestionTypeCount.class);
		return outputTypeCount.getMappedResults();
	}
	
	private static void disposeQuestion(Question qes) {
        if (qes != null) {

            //处理选项
            if (qes.getQuesOptions() != null && qes.getQuesOptions().size() > 0) {
                int index = 0;
                for (QuesOption qo : qes.getQuesOptions()) {
                    List<JSection> slist2 = getSections(qo.getOptionBody());
                    if (slist2 != null && slist2.size() > 0) {
                        SectionElement se = new SectionElement();
                        se.setType(ELEMENT_TYPE_TEXT);
                        se.setValue(QUESOPS.charAt(index) + ".");
                        index++;
                        slist2.get(0).getElements().add(0, se);
//                        qo.setOptionBodyWord(getQuestionDoc(slist2));
                    }else {
                    	System.out.println("******");
                    }

                }
            }
            
            if (qes.getSubQuestions() != null && qes.getSubQuestions().size() > 0) {
                for (Question sunqes : qes.getSubQuestions()) {
                    disposeQuestion(sunqes);// 递归处理套题
                }
            }
        }
    }
	private static List<JSection> getSections(String html) {
        return getSections(html, false);
    }

    private static List<JSection> getSections(String html, boolean diposeFillBlank) {
        if (StringUtils.isBlank(html)) {
            return getEmptySections();
        }
        html = html.trim();
        if (!html.startsWith("<p>")) {
            html = "<p>" + html + "</p>";
        }
        List<JSection> ss = new ArrayList<JSection>();
        Document doc = Jsoup.parse(html);
        Element b = doc.body();
        for (Node e : b.childNodes()) {
            if ("p".equals(e.nodeName()) && e.childNodeSize() > 0) {
                JSection s = new JSection();
                List<SectionElement> ses = new ArrayList<SectionElement>();
                s.setElements(ses);
                for (Node ce : e.childNodes()) {
                    getSectionElement(ce, ses, diposeFillBlank);
                }
                ss.add(s);
            }
        }
        return ss;

    }
    
    private static List<JSection> getEmptySections(){
    	List<JSection> ss = new ArrayList<JSection>();
    	JSection s = new JSection();
        List<SectionElement> ses = new ArrayList<SectionElement>();
        s.setElements(ses);
        SectionElement se=new SectionElement();
        se.setType(ELEMENT_TYPE_TEXT);
        se.setValue("");
        ses.add(se);
        ss.add(s);
        return ss;
    }

    private static void getSectionElement(Node ce, List<SectionElement> ses, boolean diposeFillBlank) {
        if ("span".equals(ce.nodeName()) && ce.childNodeSize() > 0) {
            for (Node e : ce.childNodes()) {
                getSectionElement(e, ses, diposeFillBlank);
            }
        } else {
            if (ce instanceof TextNode) {
            	SectionElement se = new SectionElement();
                TextNode tn = (TextNode) ce;
                se.setType(ELEMENT_TYPE_TEXT);
                String text = tn.text();
                if (diposeFillBlank) {
                    text = text.replaceAll("###", "______").replaceAll("##", "______");
                }
                text = text.replaceAll("&", "&amp;").replaceAll("<", "&lt;").replaceAll(">", "&gt;").replaceAll("\u0000", " ");
                se.setValue(text);
                ses.add(se);
            } else if (ce instanceof Element) {
            	SectionElement se = new SectionElement();
                if ("img".equals(ce.nodeName())) {
                    se.setType(ELEMENT_TYPE_IMG);
                    se.setValue(ce.attr("src"));
//                    setImageSize(se, ce);
                } else if ("br".equals(ce.nodeName())) {
                    se.setType(ELEMENT_TYPE_DOCTAG);
                    se.setValue("<w:r w:rsidRPr=\"003A4B1C\"><w:br/></w:r>");
                } else {
                    Element el = (Element) ce;
                    se.setType(ELEMENT_TYPE_TEXT);
                    se.setValue(el.text());
                }
                ses.add(se);
            }
        }
    }

}
