package cn.com.qmth.dp.examcloud.oe.modules.export_course_questions_count;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.collections4.CollectionUtils;
import org.bson.types.ObjectId;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationOperation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import cn.com.qmth.dp.examcloud.oe.entity.question.Course;
import cn.com.qmth.dp.examcloud.oe.enums.question.QuesStructType;
import cn.com.qmth.dp.examcloud.oe.multithread.Consumer;

@Service
@Scope("prototype")
public class ExportQuesBySuffConsumer extends Consumer<Course> {

    @Autowired
    private MongoTemplate mongoTemplate;

    @Override
    public void consume(Map<String, Object> param, Course c) {
        CourseQuestionsCountRetDto rd = new CourseQuestionsCountRetDto();
        String rootOrgId = (String) param.get("rootOrgId");
        String paperSuff = (String) param.get("paperSuff");
        rd.setCourseCode(c.getCode());
        rd.setCourseName(c.getName());
        List<ObjectId> pids = getPaperIds(paperSuff, c.getCode(), rootOrgId);
        if (CollectionUtils.isEmpty(pids)) {
            return;
        }
        List<QuestionTypeCount> tc = countByType(pids);
        if (CollectionUtils.isNotEmpty(tc)) {
            setCount(rd, tc);
            // setExtCount(paperSuff, rd, rootOrgId);
        }
        if (rd.getSingleAnswerQuestion() != 0 || rd.getMultipleAnswerQuestion() != 0 || rd.getBoolAnswerQuestion() != 0
                || rd.getFillBlankQuestion() != 0 || rd.getTextAnswerQuestion() != 0
                || rd.getNestedAnswerQuestion() != 0 || rd.getExt() != 0) {
            addRet(rd);
        }

    }

    private List<ObjectId> getPaperIds(String paperSuff, String courseCode, String rootOrgId) {
        Query query = new Query();
        query.addCriteria(Criteria.where("orgId").is(rootOrgId).and("paperType").is("IMPORT").and("course.code")
                .is(courseCode).and("creationBy").is(646523L));
        List<IdBean> ps = mongoTemplate.find(query, IdBean.class, "paper");
        if (CollectionUtils.isEmpty(ps)) {
            return null;
        }
        Pattern pa = Pattern.compile("\\(" + paperSuff + "\\)");
        List<ObjectId> ids = new ArrayList<>();
        for (IdBean p : ps) {
            Matcher ma = pa.matcher(p.getName());
            if (ma.find()) {
                ids.add(new ObjectId(p.getId()));
            }
        }
        return ids;
    }

    // private List<ObjectId> getDetailIds(List<ObjectId> ps) {
    // Query query = new Query();
    // query.addCriteria(Criteria.where("paper.$id").in(ps));
    // List<Detail> ds = mongoTemplate.find(query, Detail.class, "paperDetail");
    // if (CollectionUtils.isEmpty(ps)) {
    // return null;
    // }
    // List<ObjectId> ids = new ArrayList<>();
    // // for(Detail d:ds) {
    // // if(d.getName().contains("案例分析")) {
    // // ids.add(new ObjectId(d.getId()));
    // // }
    // // }
    // return ids;
    // }

    // private long getExtCount(List<ObjectId> ds) {
    // Query query = new Query();
    // query.addCriteria(Criteria.where("paperDetail.$id").in(ds).and("questionType")
    // .is(QuesStructType.TEXT_ANSWER_QUESTION.name()));
    // long count = mongoTemplate.count(query, "paperDetailUnit");
    // return count;
    // }

    // private void setExtCount(String paperSuff, CourseQuestionsCountRetDto rd,
    // String rootOrgId) {
    // List<ObjectId> paperIds = getPaperIds(paperSuff, rd.getCourseCode(),
    // rootOrgId);
    // if (CollectionUtils.isEmpty(paperIds)) {
    // return;
    // }
    //
    // List<ObjectId> detailIds = getDetailIds(paperIds);
    // if (CollectionUtils.isEmpty(detailIds)) {
    // return;
    // }
    // rd.setExt(getExtCount(detailIds));
    // }

    private void setCount(CourseQuestionsCountRetDto rd, List<QuestionTypeCount> tc) {
        for (QuestionTypeCount t : tc) {
            if (t.getQuestionType().equals(QuesStructType.SINGLE_ANSWER_QUESTION.name())) {
                rd.setSingleAnswerQuestion(t.getCount());
            } else if (t.getQuestionType().equals(QuesStructType.MULTIPLE_ANSWER_QUESTION.name())) {
                rd.setMultipleAnswerQuestion(t.getCount());
            } else if (t.getQuestionType().equals(QuesStructType.BOOL_ANSWER_QUESTION.name())) {
                rd.setBoolAnswerQuestion(t.getCount());
            } else if (t.getQuestionType().equals(QuesStructType.FILL_BLANK_QUESTION.name())) {
                rd.setFillBlankQuestion(t.getCount());
            } else if (t.getQuestionType().equals(QuesStructType.TEXT_ANSWER_QUESTION.name())) {
                rd.setTextAnswerQuestion(t.getCount());
            } else if (t.getQuestionType().equals(QuesStructType.NESTED_ANSWER_QUESTION.name())) {
                rd.setNestedAnswerQuestion(t.getCount());
            }
        }
    }

    private List<QuestionTypeCount> countByType(List<ObjectId> pids) {
        List<AggregationOperation> operations = new ArrayList<>();
        operations.add(Aggregation.match(Criteria.where("paper.$id").in(pids)));
        operations.add(Aggregation.group("questionType").count().as("count"));
        operations.add(Aggregation.project("count").and("questionType").previousOperation());
        Aggregation aggregation = Aggregation.newAggregation(operations);
        AggregationResults<QuestionTypeCount> outputTypeCount = mongoTemplate.aggregate(aggregation,
                PaperDetailUnitDto.class, QuestionTypeCount.class);
        return outputTypeCount.getMappedResults();
    }

}
