package cn.com.qmth.dp.examcloud.oe.modules.export_extract_config;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import cn.com.qmth.dp.examcloud.oe.entity.question.ExamPaper;
import cn.com.qmth.dp.examcloud.oe.entity.question.RandomPaper;
import cn.com.qmth.dp.examcloud.oe.excel.ExportUtils;
import cn.com.qmth.examcloud.api.commons.enums.CallType;
import cn.com.qmth.examcloud.commons.exception.StatusException;

/**
 * 
 * @author chenken
 *
 */
@Service
public class ExportExtractConfigService {
	private static List<Long> examIds=new ArrayList<>();
	
	static {
		examIds.add(5591L);
		examIds.add(5592L);
		examIds.add(5593L);
	}
	@Autowired
	private MongoTemplate mongoTemplate;
	
	public void start() {
		File file = new File("d:/ret.xlsx");
		if (file.exists()) {
			file.delete();
		}
		Query query = new Query();
		query.addCriteria(Criteria.where("examId").in(examIds));
		List<ExtractConfig> ps = mongoTemplate.find(query, ExtractConfig.class, "extractConfig");
		if(CollectionUtils.isEmpty(ps)) {
			return;
		}
		List<ExtractConfigDto> ret=new ArrayList<>();
		for(ExtractConfig ec:ps) {
			if(CallType.RANDOM_PAPER.equals(ec.getCallType())) {
				ExtractConfigDto dto=new ExtractConfigDto();
				dto.setExamId(ec.getExamId());
				dto.setCourseName(ec.getCourse().getName());
				dto.setCourseCode(ec.getCourse().getCode());
				dto.setCallType(ec.getCallType().getName());
				dto.setPaperName(getPaperName(ec.getRandomPaperId()));
				ret.add(dto);
			}else {
				for(ExamPaper ep:ec.getExamPaperList()) {
					ExtractConfigDto dto=new ExtractConfigDto();
					dto.setExamId(ec.getExamId());
					dto.setCourseName(ec.getCourse().getName());
					dto.setCourseCode(ec.getCourse().getCode());
					dto.setCallType(ec.getCallType().getName());
					dto.setPaperName(ep.getPaper().getName());
					ret.add(dto);
				}
			}
		}
		Collections.sort(ret, new Comparator<ExtractConfigDto>() {

            @Override
            public int compare(ExtractConfigDto o1, ExtractConfigDto o2) {
                if (o1.getExamId() < o2.getExamId()) {
                    return -1;
                } else if (o1.getExamId()> o2.getExamId()) {
                    return 1;
                } else {
                    return o1.getCourseCode().compareTo(o2.getCourseCode());
                }
            }

        });
		FileOutputStream fos = null;
		try {
			file.createNewFile();
			fos = new FileOutputStream(file);
			ExportUtils.makeExcel(ExtractConfigDto.class, ret, fos);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (fos != null) {
				try {
					fos.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	private String getPaperName(String id) {
		RandomPaper p = mongoTemplate.findById(id, RandomPaper.class);
		if(p==null) {
			throw new StatusException("未找到千卷模版");
		}
		return p.getName();
	}
}
