package cn.com.qmth.dp.examcloud.oe.modules.findcheatanswer;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import cn.com.qmth.dp.examcloud.oe.entity.question.ExamQuestionEntity;
import cn.com.qmth.dp.examcloud.oe.entity.question.ExamRecordQuestionsEntity;
import cn.com.qmth.dp.examcloud.oe.multithread.Consumer;
import cn.com.qmth.dp.examcloud.oe.util.PaperUtil;
import cn.com.qmth.dp.examcloud.oe.util.StringSimilarityUtils;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.question.commons.core.question.QuestionType;

@Service
@Scope("prototype")
public class FindCheatAnswerConsumer extends Consumer<Long> {
	@Autowired
	private FindCheatAnswerService findCheatAnswerService;

	@Resource(name = "secondaryMongoTemplate")
	private MongoTemplate oeTemplate;

	@Override
	public void consume(Map<String, Object> param, Long recordId) {
		ExamRecordQuestionsEntity qs = getStudentAnswer(recordId);
		if (qs == null) {
			throw new StatusException("未找到考生作答:"+recordId);
		}
		Map<String, Integer> subIndex = new HashMap<>();
		for (ExamQuestionEntity q : qs.getExamQuestionEntities()) {
			Integer curIndex = subIndex.get(q.getQuestionId());
			if (curIndex == null) {
				subIndex.put(q.getQuestionId(), 0);
			} else {
				subIndex.put(q.getQuestionId(), curIndex + 1);
			}
			if (QuestionType.SINGLE_CHOICE.equals(q.getQuestionType())
					|| QuestionType.TRUE_OR_FALSE.equals(q.getQuestionType())) {
//				if (!StringUtils.equals(q.getCorrectAnswer(), q.getStudentAnswer())) {
//					return;
//				}
			} else if (QuestionType.MULTIPLE_CHOICE.equals(q.getQuestionType())) {
//				if (!StringUtils.equals(sort(q.getCorrectAnswer()), sort(q.getStudentAnswer()))) {
//					return;
//				}
			} else if (QuestionType.FILL_UP.equals(q.getQuestionType())) {
				String correctAnswer = findCheatAnswerService.getQuestionAnswerText(q.getQuestionId(),
						subIndex.get(q.getQuestionId()));
				if (!StringUtils.equals(dis(q.getStudentAnswer()), correctAnswer)) {
					return;
				}
			} else if (QuestionType.ESSAY.equals(q.getQuestionType())) {
				String correctAnswer = findCheatAnswerService.getQuestionAnswerText(q.getQuestionId(),
						subIndex.get(q.getQuestionId()));
				String studentAnswer=PaperUtil.getExtractText(q.getStudentAnswer());
				double similarity = StringSimilarityUtils.getSimilarityWithCosinesBySeg(correctAnswer, studentAnswer);
				if (similarity < 0.85) {
					return;
				}
			}else {
				throw new StatusException("试题类型错误:"+recordId+" "+q.getOrder());
			}
		}
		addRet(qs.getExamRecordDataId());
	}

	private String dis(String s) {
		if (StringUtils.isBlank(s)) {
			return "";
		}
		return s.replaceAll("##", "");
	}

	private ExamRecordQuestionsEntity getStudentAnswer(Long recordId) {
		Query query = new Query();
		query.addCriteria(Criteria.where("examRecordDataId").is(recordId));
		List<ExamRecordQuestionsEntity> ret = oeTemplate.find(query, ExamRecordQuestionsEntity.class,
				"examRecordQuestions");
		if(CollectionUtils.isEmpty(ret)) {
			return null;
		}
		return ret.get(0);
	}

	private String sort(String s) {
		if (StringUtils.isBlank(s)) {
			return "";
		}
		char[] stringtoChar = s.toCharArray();
		Arrays.sort(stringtoChar);
		String sortedString = new String(stringtoChar);
		return sortedString;
	}
}
